"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CertificateEventPostgresRepository = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const typeorm_2 = require("typeorm");
const array_1 = require("../../utils/array");
const CertificateEvent_entity_1 = require("./CertificateEvent.entity");
const Certificate_events_1 = require("../../events/Certificate.events");
const CertificateSynchronizationAttempt_entity_1 = require("./CertificateSynchronizationAttempt.entity");
const config_1 = require("@nestjs/config");
let CertificateEventPostgresRepository = class CertificateEventPostgresRepository {
    constructor(repository, synchronizationAttemptRepository, configService) {
        this.repository = repository;
        this.synchronizationAttemptRepository = synchronizationAttemptRepository;
        this.maxSynchronizationAttemptsForEvent = configService.get('MAX_SYNCHRONIZATION_ATTEMPTS_FOR_EVENT');
    }
    async getAll() {
        return await this.repository.find();
    }
    async getByInternalCertificateId(internalCertId) {
        return await this.repository.find({
            where: {
                internalCertificateId: internalCertId
            }
        });
    }
    async save(event, commandId, txManager) {
        const repository = txManager
            ? txManager.getRepository(CertificateEvent_entity_1.CertificateEventEntity)
            : this.repository;
        return await repository.save({
            internalCertificateId: event.internalCertificateId,
            type: event.type,
            version: event.version,
            payload: event.payload,
            createdAt: event.createdAt,
            commandId: commandId
        });
    }
    async createSynchronizationAttempt(eventId, txManager) {
        const repository = txManager
            ? txManager.getRepository(CertificateSynchronizationAttempt_entity_1.CertificateSynchronizationAttemptEntity)
            : this.synchronizationAttemptRepository;
        return await repository.save({
            eventId,
            attemptsCount: 0
        });
    }
    async updateAttempt({ eventId, error }) {
        const synchronizationAttempt = await this.synchronizationAttemptRepository.findOne(eventId);
        if (!synchronizationAttempt) {
            throw new Error('Synchronization attempt does not exist');
        }
        synchronizationAttempt.attemptsCount = synchronizationAttempt.attemptsCount + 1;
        synchronizationAttempt.error = error ?? null;
        await this.synchronizationAttemptRepository.update(synchronizationAttempt.eventId, synchronizationAttempt);
        return synchronizationAttempt;
    }
    async findAllToProcess(params) {
        const failedCertificatesIds = this.repository
            .createQueryBuilder('event')
            .select(['event.internalCertificateId'])
            .innerJoin(CertificateSynchronizationAttempt_entity_1.CertificateSynchronizationAttemptEntity, 'attempt', 'attempt.eventId = event.id')
            .where(`(attempt.attempts_count >= :attempts)`, {
            attempts: this.maxSynchronizationAttemptsForEvent
        });
        const query = this.repository
            .createQueryBuilder('event')
            .select([
            'event.id',
            'event.internalCertificateId',
            'event.createdAt',
            'event.type',
            'event.version',
            'event.payload'
        ])
            .innerJoinAndSelect(CertificateSynchronizationAttempt_entity_1.CertificateSynchronizationAttemptEntity, 'attempt', 'attempt.eventId = event.id')
            .where(`
                (
                    (attempt.attempts_count = 0) 
                    OR 
                    (attempt.error IS NOT NULL AND attempt.attempts_count < :attempts)
                )`, { attempts: this.maxSynchronizationAttemptsForEvent })
            .andWhere(`event.internalCertificateId NOT IN (${failedCertificatesIds.getQuery()})`)
            .orderBy('event."createdAt"');
        if (params.limit) {
            return await query.limit(params.limit).getMany();
        }
        else {
            return await query.getMany();
        }
    }
    async getOne(eventId) {
        const event = await this.repository.findOne(eventId);
        if (!event) {
            throw new common_1.NotFoundException();
        }
        return event;
    }
    async getBlockchainIdMap(internalCertIds) {
        const events = (await this.repository.find({
            where: {
                type: Certificate_events_1.CertificateEventType.IssuancePersisted,
                internalCertificateId: typeorm_2.In(internalCertIds)
            }
        }));
        return array_1.arrayToMap(events, (e) => e.internalCertificateId, (e) => e.payload.blockchainCertificateId);
    }
    async getNumberOfCertificates() {
        const [certs, count] = await this.repository.findAndCount({
            where: {
                type: Certificate_events_1.CertificateEventType.Issued
            }
        });
        return count;
    }
};
CertificateEventPostgresRepository = __decorate([
    common_1.Injectable(),
    __param(0, typeorm_1.InjectRepository(CertificateEvent_entity_1.CertificateEventEntity)),
    __param(1, typeorm_1.InjectRepository(CertificateSynchronizationAttempt_entity_1.CertificateSynchronizationAttemptEntity)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        config_1.ConfigService])
], CertificateEventPostgresRepository);
exports.CertificateEventPostgresRepository = CertificateEventPostgresRepository;
//# sourceMappingURL=CertificateEventPostgres.repository.js.map