"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var BatchSynchronizeStrategy_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.BatchSynchronizeStrategy = void 0;
const common_1 = require("@nestjs/common");
const events_batch_queue_1 = require("./events.batch-queue");
const Certificate_events_1 = require("../../../events/Certificate.events");
const repository_keys_1 = require("../../../repositories/repository.keys");
const claim_persist_handler_1 = require("../../handlers/claim-persist.handler");
const issue_persist_handler_1 = require("../../handlers/issue-persist.handler");
const transfer_persist_handler_1 = require("../../handlers/transfer-persist.handler");
const lodash_1 = require("lodash");
let BatchSynchronizeStrategy = BatchSynchronizeStrategy_1 = class BatchSynchronizeStrategy {
    constructor(eventRepository, claimPersistHandler, issuancePersistHandler, transferPersistHandler) {
        this.eventRepository = eventRepository;
        this.claimPersistHandler = claimPersistHandler;
        this.issuancePersistHandler = issuancePersistHandler;
        this.transferPersistHandler = transferPersistHandler;
        this.logger = new common_1.Logger(BatchSynchronizeStrategy_1.name);
    }
    async synchronize(events) {
        const failedCertificateIds = [];
        const hasCertificateFailed = (certificateId) => failedCertificateIds.includes(certificateId);
        const eventsBatchQueue = new events_batch_queue_1.EventsBatchQueue(events);
        while (!eventsBatchQueue.isEmpty()) {
            const batch = eventsBatchQueue.popBatch();
            const issueEvents = batch.filter(Certificate_events_1.isIssueEvent);
            const transferEvents = batch.filter(Certificate_events_1.isTransferEvent);
            const claimEvents = batch.filter(Certificate_events_1.isClaimEvent);
            const issueResult = await this.issuancePersistHandler.handleBatch(issueEvents.filter((e) => !hasCertificateFailed(e.internalCertificateId)));
            failedCertificateIds.push(...issueResult.failedCertificateIds);
            eventsBatchQueue.removeEventsForCertificateIds(issueResult.failedCertificateIds);
            const { events: transferEventsWithBlockchainId, idMap: transferIdMap } = await this.getEventsWithBlockchainId(transferEvents.filter((e) => !hasCertificateFailed(e.internalCertificateId)));
            const transferResult = await this.transferPersistHandler.handleBatch(transferEventsWithBlockchainId, transferIdMap);
            eventsBatchQueue.removeEventsForCertificateIds(transferResult.failedCertificateIds);
            const { events: claimEventsWithBlockchainId, idMap: claimIdMap } = await this.getEventsWithBlockchainId(claimEvents.filter((e) => !hasCertificateFailed(e.internalCertificateId)));
            const claimResult = await this.claimPersistHandler.handleBatch(claimEventsWithBlockchainId, claimIdMap);
            failedCertificateIds.push(...claimResult.failedCertificateIds);
            eventsBatchQueue.removeEventsForCertificateIds(claimResult.failedCertificateIds);
        }
    }
    async getEventsWithBlockchainId(events) {
        const idMap = await this.eventRepository.getBlockchainIdMap(events.map((e) => e.internalCertificateId));
        const [eventsWithBlockchainId, eventsWithoutBlockchainId] = lodash_1.partition(events, (e) => idMap[e.internalCertificateId] !== undefined);
        if (eventsWithoutBlockchainId.length > 0) {
            this.logger.error(`Cannot synchronize events: ${eventsWithoutBlockchainId
                .map((e) => e.id)
                .join(', ')}, because they don't have blockchain ID`);
        }
        return {
            events: eventsWithBlockchainId,
            idMap
        };
    }
};
BatchSynchronizeStrategy = BatchSynchronizeStrategy_1 = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject(repository_keys_1.CERTIFICATE_EVENT_REPOSITORY)),
    __metadata("design:paramtypes", [Object, claim_persist_handler_1.ClaimPersistHandler,
        issue_persist_handler_1.IssuePersistHandler,
        transfer_persist_handler_1.TransferPersistHandler])
], BatchSynchronizeStrategy);
exports.BatchSynchronizeStrategy = BatchSynchronizeStrategy;
//# sourceMappingURL=batch-synchronize.strategy.js.map