import { OnChainCertificateService } from './onchain-certificate.service';
import { IGetAllCertificatesOptions } from '@energyweb/issuer-api';
import { ICertificate } from './types';
import { IClaimCommand, IIssueCommandParams, IIssuedCertificate, ITransferCommand } from '../types';
declare type PublicPart<T> = {
    [K in keyof T]: T[K];
};
export declare class CertificateForUnitTestsService<T> implements PublicPart<OnChainCertificateService<T>> {
    protected serial: number;
    private db;
    getAll(options?: IGetAllCertificatesOptions): Promise<ICertificate<T>[]>;
    getById(id: number): Promise<ICertificate<T> | null>;
    issue(params: IIssueCommandParams<T>): Promise<IIssuedCertificate<T>>;
    claim(command: IClaimCommand): Promise<void>;
    transfer(command: ITransferCommand): Promise<void>;
    batchIssue(originalCertificates: IIssueCommandParams<T>[]): Promise<number[]>;
    batchClaim(command: IClaimCommand[]): Promise<void>;
    batchTransfer(command: ITransferCommand[]): Promise<void>;
    batchIssueWithTxHash(command: IIssueCommandParams<T>[]): Promise<{
        certificateIds: number[];
        transactionHash: string;
    }>;
    batchTransferWithTxHash(command: ITransferCommand[]): Promise<{
        transactionHash: string;
    }>;
    batchClaimWithTxHash(command: IClaimCommand[]): Promise<{
        transactionHash: string;
    }>;
    issueWithTxHash<T>(command: IIssueCommandParams<T>): Promise<{
        certificate: IIssuedCertificate<T>;
        transactionHash: string;
    }>;
    transferWithTxHash(command: ITransferCommand): Promise<{
        transactionHash: string;
    }>;
    claimWithTxHash(command: IClaimCommand): Promise<{
        transactionHash: string;
    }>;
}
export {};
